codeunit 70030 "Intercept Ship Lines SBIESS"
{

    // This procedure is used by events to find out if there are multiple "Ship-To" codes used
    // on the same Sales Order.
    local procedure CheckMultipleShipTos(var SalesHeader: Record "Sales Header"; var TempShipTo: Record "Ship-to Address" temporary)
    var
        SalesLine: Record "Sales Line";
    begin
        SalesLine.SetRange("Document Type", SalesHeader."Document Type");
        SalesLine.SetRange("Document No.", SalesHeader."No.");
        // This is a key part, otherwise we'll create empty Sales Shipment Headers for Ship-to's we're not handling.
        SalesLine.SetFilter("Qty. to Ship", '<>0');
        if SalesLine.FindSet(false) then
            repeat
                if not TempShipTo.Get(SalesHeader."Sell-to Customer No.", SalesLine."Ship-To Code SBIESS") then begin
                    TempShipTo."Customer No." := SalesHeader."Sell-to Customer No.";
                    TempShipTo.Code := SalesLine."Ship-To Code SBIESS";
                    TempShipTo.Insert(false);
                    // This enables us to have a list of Unique Ship-Tos.  
                    // We could have also used a List datatype to similar effect.
                end;
            until SalesLine.Next() = 0;
    end;

    // This procedure populates the Sales Shipment Header in the same way as the Sales Post
    // codeunit.
    local procedure UpdateSalesShipmentHeaderWithShipTo(var SalesShipmentHeader: Record "Sales Shipment Header"; Code: Code[10])
    var
        ShipToAddr: Record "Ship-to Address";
    begin
        SalesShipmentHeader."Ship-to Name" := ShipToAddr.Name;
        SalesShipmentHeader."Ship-to Name 2" := ShipToAddr."Name 2";
        SalesShipmentHeader."Ship-to Address" := ShipToAddr.Address;
        SalesShipmentHeader."Ship-to Address 2" := ShipToAddr."Address 2";
        SalesShipmentHeader."Ship-to City" := ShipToAddr.City;
        SalesShipmentHeader."Ship-to Post Code" := ShipToAddr."Post Code";
        SalesShipmentHeader."Ship-to County" := ShipToAddr.County;
        SalesShipmentHeader.Validate("Ship-to Country/Region Code", ShipToAddr."Country/Region Code");
        SalesShipmentHeader."Ship-to Contact" := ShipToAddr.Contact;
        if ShipToAddr."Shipment Method Code" <> '' then
            SalesShipmentHeader.Validate("Shipment Method Code", ShipToAddr."Shipment Method Code");
        if ShipToAddr."Location Code" <> '' then
            SalesShipmentHeader.Validate("Location Code", ShipToAddr."Location Code");
        SalesShipmentHeader."Shipping Agent Code" := ShipToAddr."Shipping Agent Code";
        SalesShipmentHeader."Shipping Agent Service Code" := ShipToAddr."Shipping Agent Service Code";
        if ShipToAddr."Tax Area Code" <> '' then
            SalesShipmentHeader."Tax Area Code" := ShipToAddr."Tax Area Code";
        SalesShipmentHeader."Tax Liable" := ShipToAddr."Tax Liable";
    end;



    // This event watches the Posting process for the Sales Posting, and at the moment it creates the 'standard' Sales
    // Shipment Header (posted sales shipment), it creates additional Sales Shipment Header
    [EventSubscriber(ObjectType::Codeunit, Codeunit::"Sales-Post", 'OnAfterSalesShptHeaderInsert', '', false, false)]
    local procedure AfterMainShipmentCreated(var SalesShipmentHeader: Record "Sales Shipment Header"; SalesHeader: Record "Sales Header"; SuppressCommit: Boolean; WhseShip: Boolean; WhseReceive: Boolean; var TempWhseShptHeader: Record "Warehouse Shipment Header"; var TempWhseRcptHeader: Record "Warehouse Receipt Header")
    var
        SalesShipmentHeader2: Record "Sales Shipment Header";
        TempShipTo: Record "Ship-to Address" temporary;
        NoSeriesMgt: Codeunit NoSeriesManagement;
        i: Integer;
    begin
        // Built a count of *unique* ship-to codes involved in this shipment posting
        CheckMultipleShipTos(SalesHeader, TempShipTo);

        // We *only* need to do special processing if there are multiple ship-tos (blank included!)
        if TempShipTo.Count() > 1 then begin
            TempShipTo.FindFirst();  // No error handling necessary, count made sure there is a record.

            //For the first record, we tag the 'standard shipment' with the ship-to code in our buffer
            SalesShipmentHeader."Split Main Shipment SBIESS" := SalesShipmentHeader."No.";
            SalesShipmentHeader."Split Ship Ship-To SBIESS" := TempShipTo.Code;
            // Because of partial shipping, we *might* need to update the Sales Shipment Header with the Ship-To information.
            if TempShipTo.Code <> '' then
                UpdateSalesShipmentHeaderWithShipTo(SalesShipmentHeader, TempShipTo.Code);
            SalesShipmentHeader.Modify(true);
            TempShipTo.Next();

            // For subsequent Ship-Tos, we create new shipment headers
            repeat
                SalesShipmentHeader2.TransferFields(SalesShipmentHeader);
                SalesShipmentHeader2."No." := NoSeriesMgt.GetNextNo(SalesHeader."Shipping No. Series", SalesHeader."Posting Date", true);
                SalesShipmentHeader2."Split Main Shipment SBIESS" := SalesShipmentHeader."No.";
                SalesShipmentHeader2."Split Ship Ship-To SBIESS" := TempShipTo.Code;
                if TempShipTo.Code <> '' then
                    UpdateSalesShipmentHeaderWithShipTo(SalesShipmentHeader, TempShipTo.Code);
                SalesShipmentHeader2.Insert(true);
            until TempShipTo.Next() = 0;
        end;

    end;

    [EventSubscriber(ObjectType::Table, Database::"Sales Shipment Line", 'OnAfterInitFromSalesLine', '', false, false)]
    local procedure RedirectShipmentLine(SalesShptHeader: Record "Sales Shipment Header"; SalesLine: Record "Sales Line"; var SalesShptLine: Record "Sales Shipment Line")
    var
        SalesShipmentHeader2: Record "Sales Shipment Header";
    begin
        // When the Posting goes to move to the Sales Shipment Line, we re-direct if there's a Ship-To Code
        if (SalesLine."Ship-To Code SBIESS" = '') then
            exit;

        SalesShipmentHeader2.SetRange("Split Main Shipment SBIESS", SalesShptHeader."No.");  // Make sure it's related to this posting
        SalesShipmentHeader2.SetRange("Split Ship Ship-To SBIESS", SalesLine."Ship-To Code SBIESS");
        if SalesShipmentHeader2.FindFirst() then begin
            // we pull a quick 'redirect' move and make the line attached to an entirely different shipment document, tada!
            SalesShptLine."Document No." := SalesShipmentHeader2."No.";
        end;
    end;


    // This event is about making sure we have the Item Ledger Entries (and thus Value Entries)
    // are connected to the right Sales Shipment No.
    [EventSubscriber(ObjectType::Codeunit, Codeunit::"Sales-Post", 'OnPostItemJnlLineOnAfterCopyDocumentFields', '', false, false)]
    local procedure ChangeDocumentNoOnItemJnlLine(var ItemJournalLine: Record "Item Journal Line"; SalesLine: Record "Sales Line"; WarehouseReceiptHeader: Record "Warehouse Receipt Header"; WarehouseShipmentHeader: Record "Warehouse Shipment Header")
    var
        SalesShipmentHeader2: Record "Sales Shipment Header";
    begin
        // If there's no other Ship-To Code on the line, then we can skip this
        if (SalesLine."Ship-To Code SBIESS" = '') then
            exit;

        // If we're not posting a Sales Shipment Item Journal line, then we can skip this too
        if (ItemJournalLine."Document Type" <> ItemJournalLine."Document Type"::"Sales Shipment") then
            exit;

        // Redirect the Item Journal Line to the correct 'split' shipment Document No.
        SalesShipmentHeader2.SetRange("Split Main Shipment SBIESS", ItemJournalLine."Document No.");
        SalesShipmentHeader2.SetRange("Split Ship Ship-To SBIESS", SalesLine."Ship-To Code SBIESS");
        if SalesShipmentHeader2.FindFirst() then begin
            ItemJournalLine."Document No." := SalesShipmentHeader2."No.";
        end;
    end;
}
